<?php

namespace Drupal\monit_drupal_connector\Plugin\HealthCheck;

use Drupal\Core\State\StateInterface;
use Drupal\monit_drupal_connector\Plugin\HealthCheckPluginBase;
use Drupal\Core\Link;
use Drupal\security_review\CheckResult;

/**
 * @HealthCheck(
 *   id = "monit_security_review",
 *   label = @Translation("Monit Security Review"),
 *   description = @Translation("Gathering Security Review data.")
 * )
 */
class MonitSecurityReview extends HealthCheckPluginBase {

    /**
     * The current plugin ID.
     *
     * @var string $pluginId
     */
    protected $pluginId;
    /**
     * The Security Review manager service.
     *
     * @var \Drupal\security_review\SecurityReview
     */
    protected $securityReview;

    /**
     * The state storage.
     *
     * @var \Drupal\Core\State\StateInterface
     */
    protected StateInterface $state;

    /**
     * The Security Review manager service.
     *
     * @var \Drupal\security_review\SecurityReviewManager
     */
    protected $securityReviewPluginManager;

    /**
     * Constructs a MonitSecurityReview object.
     */
    public function __construct(array $configuration, $plugin_id, $plugin_definition) {
        $this->securityReview = \Drupal::service('security_review');
        $this->state = \Drupal::service('state');
        $this->securityReviewPluginManager = \Drupal::service('plugin.manager.security_review.security_check');
        $this->pluginId = $plugin_id;
    }

    /**
     * Set the Health Check label.
     *
     * @return string
     */
    public function label()
    {
        return 'Security Review Health Check Data';
    }

    /**
     * Set the Health Check descriptiion.
     *
     * @return string
     */
    public function description()
    {
        return 'This health check will return all the findings of the security review module.';
    }

    /**
     * Set the payload data.
     *
     * @return array $payload
     */
    public function data()
    {

        //BUG FIX FOR EMPTY DATA
        // Determine web server's uid and groups.
        $uid = posix_getuid();
        $groups = posix_getgroups();

        // Store the data in the State system.
        $this->state->set('security_review.server.uid', $uid ?: -1);
        $this->state->set('security_review.server.groups', $groups ?: []);

        $checks = $this->securityReviewPluginManager->getChecks();
        $this->securityReview->runChecks($checks);
        $this->securityReview->setLastRun(time());
        $definitions = $this->securityReviewPluginManager->getDefinitions();
        $data = [];
        $helpDetails = [];
        foreach ($definitions as $id => $definition) {
            $plugin = $this->securityReviewPluginManager->createInstance($id);
            $lastResult = $plugin->lastResult();
            $result_number = $lastResult['result'];
            $details = [];
            $resultStatus = '';
            switch ($result_number) {
                case CheckResult::SUCCESS:
                    $resultStatus = 'success';
                    break;

                case CheckResult::FAIL:
                    $resultStatus = 'fail';
                    break;

                case CheckResult::WARN:
                    $resultStatus = 'warning';
                    break;

                case CheckResult::INFO:
                    $resultStatus = 'info';
                    break;
            }
            $resultMessage = $plugin->getStatusMessage($result_number);
            $resultDetails = $plugin->getDetails($lastResult['findings'], $lastResult['hushed']);
            foreach ($resultDetails as $resultDetail) {
                foreach ($resultDetail['#paragraphs'] as $paragraph) {
                    if ($paragraph instanceof Link) {
                        $details[] = [
                            "type" => "link",
                            "value" => $paragraph->toString()
                        ];
                    }
                    else {
                        $details[] = [
                            "type" => "paragraph",
                            "value" => $paragraph->render()
                        ];
                    }
                }
                if (isset($resultDetail['#items'])) {
                    $items = [];
                    foreach ($resultDetail['#items'] as $item) {
                        $items[] = $item;
                    }
                    $details[] = [
                        "type" => "list",
                        "items" => $items
                    ];
                }
            }

            $help_text = $plugin->getHelp();
            foreach ($help_text['#paragraphs'] as $paragraph) {
                $helpDetails[] = $paragraph->render();
            }
            $data[] = [
                'id' => $this->pluginId . '_' . $id,
                'label' => $plugin->getTitle(),
                'description' => $plugin->getDescription(),
                'status' => $resultStatus,
                'statusDescription' => 'a string that indicates the success label',
                'namespace' => $plugin->getNamespace(),
                'detailsTitle' => $resultMessage,
                'detailsText' => $helpDetails,
                'detailsFindings' => $details,
                'detailsExtra' => [
                  'findings' => array_merge($lastResult['findings'], $lastResult['hushed']),
                ]
            ];
        }

        return $data;
    }

}
